package lols.auth.ri;

import lols.auth.api.*;
import lols.core.common.DomainResolver;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.Random;

public class ContextAuthService implements AuthService {

    private final DomainResolver domainResolver;

    private final Map<String, String> loginPasswordMappings;
    private final Map<String, String> userDomainMappings;

    private final Map<String, String> tokenRepository = new HashMap<String, String>();

    public ContextAuthService(DomainResolver domainResolver, Map<String, String> loginPasswordMappings, Map<String, String> userDomainMappings) {
        this.domainResolver = domainResolver;
        this.loginPasswordMappings = loginPasswordMappings;
        this.userDomainMappings = userDomainMappings;
    }

    public String login(Credentials credentials) {
        if (credentials instanceof LoginPasswordCredentials) {
            LoginPasswordCredentials loginPasswordCredentials = (LoginPasswordCredentials) credentials;
            String login = loginPasswordCredentials.getLogin();
            String userPassword = loginPasswordMappings.get(login);

            if (userPassword == null) {
                throw new NoSuchUserException();
            }
            if (userPassword.equals(loginPasswordCredentials.getPassword())) {
                String domain = userDomainMappings.get(login);
                if (domain == null || domain.isEmpty()) {
                    throw new IllegalStateException("User domain cannot be empty.");
                }
                String resolvedDomain = domainResolver.resolveDomain();
                if (!resolvedDomain.equals(domain)) {
                    throw new DomainMismatchException();
                }
                String token = "" + new Random().nextInt(10000) + new Date().getTime();
                tokenRepository.put(token, login);
                return token;
            }
            throw new InvalidPasswordException();
        }
        throw new UnknownAuthMethodException();
    }

    public boolean isLogged(String token) {
        return tokenRepository.containsKey(token);
    }

    @Override
    public void logout(String token) {
        if (tokenRepository.remove(token) == null) {
            throw new IllegalArgumentException("No such token registered: " + token);
        }
    }

    @Override
    public String getUserDomain(String token) {
        return userDomainMappings.get(tokenRepository.get(token));
    }

}
